//
// RayDialer Goertzel Simulation
// (c)2010, ASkr
//


//**********************************************
//*** w()
//*** Creates a PCM sample in memory.
//*** 
//*** f        -> signal frequency
//*** sfreq    -> sample frequency
//*** duration -> duration in s
//*** asig     -> signal amplitude
//*** anoise   -> noise amplitude
//**********************************************
function r=w(f,sfreq,duration,asig,anoise,offset)
  t=soundsec(duration,sfreq);
  r=offset+asig*sin(2*%pi*f *t)
  r=r+anoise*rand(1,size(t,2));
endfunction


//**********************************************
//*** r()
//*** Original Goerzel algorithm
//***
//*** f -> frequency to analyze
//*** x -> array with samples of size(1,n)
//**********************************************
function r=g(f,x)
  f=f/SFREQ;
  c=16384*2*cos(2*%pi*f);
  c=int(c);
  s1=0;
  s2=0;
  for i=1:size(x,2)
    s=x(i)+int((c*s1))/16384-s2;
    s2=s1;
    s1=s;
  end
  r = (s1**2 + s2**2 - s1*s2*c/16384)/(SFREQ*SDURATION);
endfunction



//**********************************************
//**********************************************
//**********************************************


SFREQ=5000
SDURATION=0.2


sig1=w(697,SFREQ,SDURATION,300,5,0.0);
sig2=w(770,SFREQ,SDURATION,300,5,0.0);
sig3=w(852,SFREQ,SDURATION,300,5,0.0);
sig4=w(941,SFREQ,SDURATION,300,5,0.0);

sig=sig1+sig2+sig3+sig4;
sig=int(sig/4+512);


r=[];
v=[];
i=1;
clear fi;
clear f;
clear p;


for fi=linspace(0,SFREQ/2,2000)
  p(1,i)=g(fi,sig);
  f(1,i)=fi;
  i=i+1;
end


subplot(211)
plot(f,p)
subplot(212)
plot(f,20*log10(p/max(p)))




